/* vim: set ai et ts=4 sw=4: */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>

#include "about.h"
#include "liteamp.h"
#include "util.h"


GtkWidget* about_dlg_new(void)
{
    const gchar *authors[] = {
        "Dongsu Jang <iolo@hellocity.net>",
        "Jacojang <jacojang@korea.com>",
        NULL
    };
    const gchar *documenters[] = { NULL };
    gchar *translators = _("translator_credits");

    GtkWidget* dlg;
    GdkPixbuf* logo_pb;
    GdkPixbuf* icon_pb;

    if (!strcmp(translators, "translator_credits"))
        translators = NULL;

    logo_pb = gdk_pixbuf_new_from_file(PIXMAP_PATH("/liteamp-logo.png"), NULL);

    dlg = gnome_about_new("liteamp", VERSION,
        _("(C) 2002 Dongsu Jang"),
        _("A Light-Weight Music Player for GNOME"),
        authors, documenters, translators,
        logo_pb);

    icon_pb = gdk_pixbuf_new_from_file(PIXMAP_PATH("/liteamp.png"), NULL);
    if (icon_pb) {
        gtk_window_set_icon(GTK_WINDOW(dlg), icon_pb);
        gdk_pixbuf_unref(icon_pb);
    }

    return dlg;
}


void about_dlg_show(void)
{
    static GtkWidget* dlg = NULL;

    if(dlg != NULL) {
        g_assert(GTK_WIDGET_REALIZED(dlg));
        gdk_window_show(dlg->window);
        gdk_window_raise(dlg->window);
    } else {        
        dlg = about_dlg_new();

		g_signal_connect(
				G_OBJECT(dlg),
				"destroy",
				G_CALLBACK(gtk_widget_destroyed),
				&dlg);

        gtk_widget_show_all(dlg);
    }
}

/*about.c*/
