/* vim: set ai et ts=4 sw=4: */

#ifndef __liteamp_h__
#define __liteamp_h__

#include "mixer.h"
#include "prefs.h"
#include "sidebar.h"
#include "playlist.h"
#include "playback.h"
#include "decoder.h"

typedef struct _Liteamp {
	GnomeProgram* program;

    GtkWidget* app;
    GtkWidget* toolbar;
    GtkWidget* statusbar;

    // volume control on toolbar
    GtkWidget* vol_hb;
    GtkWidget* vol_eb;
    GtkWidget* vol_img;
    GtkWidget* vol_hs;

    // play info
    GtkWidget* play_info;
    GtkWidget* info_lbl;
    GtkWidget* info_eb;

    // position control
    GtkWidget* play_pos;
    GtkWidget* pos_hs;

	// sidebar
    Sidebar* sidebar;
    GtkWidget* sidebar_sw;
    GtkWidget* sidebar_vp;
    GtkWidget* sidebar_popup_menu;

    // playlist
    Playlist* playlist;
    GtkWidget* playlist_sw;
    GtkWidget* playlist_popup_menu;

    GtkWidget* right_vb;
    GtkWidget* contents_hp;

    Mixer* mixer;
	Decoder* decoder;
	Playback* playback;

    gboolean update_pos_flag;
    gboolean update_vol_flag;

    enum _PlayStatus {
        PLAY_STATUS_PLAY,
        PLAY_STATUS_PAUSE,
        PLAY_STATUS_STOP,
    } play_status;
} Liteamp;

/** global application object */
extern Liteamp liteamp;

void liteamp_init(GnomeProgram* program);
void liteamp_main(void);
void liteamp_quit(void);

GtkWindow* liteamp_get_app_window(void);
Sidebar* liteamp_get_sidebar(void);
Playlist* liteamp_get_playlist(void);

void liteamp_read_tracks_all(void);
void liteamp_read_tracks(SidebarItem* item);

void liteamp_update_ui_play_pos(void);
void liteamp_update_ui_play_status(void);
void liteamp_update_ui_play_info(const gchar* filename);
void liteamp_update_ui_play_shuffle(void);
void liteamp_update_ui_play_loop(void);
void liteamp_update_ui_play_mute(void);
void liteamp_update_ui_play_effect(void);
void liteamp_update_ui_layout(void);
void liteamp_update_ui_menu(void);
void liteamp_update_ui_status(void);
void liteamp_update_ui_volume(void);

#define PIXMAP_PATH(filename) (PIXMAPS_DIR filename)

#endif /*liteamp.h*/
