/* vim: set ai et ts=4 sw=4: */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>

#include "liteamp.h"
#include "uiinfo.h"
#include "callbacks.h"

/*-----------------------------------------------------------------*/

// main menu

GnomeUIInfo file_menu_uiinfo[] = {
    { GNOME_APP_UI_ITEM, N_("_New Playlist"), NULL, file_new_playlist_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_ADD, GDK_N, GDK_CONTROL_MASK, NULL },
    { GNOME_APP_UI_ITEM, N_("_Remove Playlist"), NULL, file_remove_playlist_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_REMOVE, 0, 0, NULL },
    GNOMEUIINFO_SEPARATOR,
    { GNOME_APP_UI_ITEM, N_("Add _Files..."), NULL, file_add_file_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/addfile.png"), GDK_F, GDK_CONTROL_MASK, NULL },
    { GNOME_APP_UI_ITEM, N_("Add _Directories..."), NULL, file_add_dir_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/adddir.png"), GDK_D, GDK_CONTROL_MASK, NULL },
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_MENU_EXIT_ITEM(file_quit_activate_cb, NULL),
    GNOMEUIINFO_END
};

GnomeUIInfo edit_menu_uiinfo[] = {
    GNOMEUIINFO_MENU_CUT_ITEM(edit_cut_activate_cb, NULL),
    GNOMEUIINFO_MENU_COPY_ITEM(edit_copy_activate_cb, NULL),
    GNOMEUIINFO_MENU_PASTE_ITEM(edit_paste_activate_cb, NULL),
    { GNOME_APP_UI_ITEM, N_("_Delete"), NULL, edit_clear_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_CLEAR, GDK_Delete, 0, NULL },
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM(N_("Select _All"), NULL, edit_select_all_activate_cb, NULL),
    GNOMEUIINFO_ITEM(N_("Select _None"), NULL, edit_select_none_activate_cb, NULL),
    GNOMEUIINFO_ITEM(N_("_Invert Selection"), NULL, edit_invert_selection_activate_cb, NULL),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_MENU_PROPERTIES_ITEM(edit_props_activate_cb, NULL),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_MENU_PREFERENCES_ITEM(edit_prefs_activate_cb, NULL),
    { GNOME_APP_UI_ITEM, N_("_Effects..."), NULL, edit_effect_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, GDK_E, GDK_CONTROL_MASK, NULL },
    GNOMEUIINFO_END
};

GnomeUIInfo play_menu_uiinfo[] = {
    { GNOME_APP_UI_ITEM, N_("_Previous"), NULL, play_prev_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/prev.png"), GDK_Z, 0, NULL },
    { GNOME_APP_UI_ITEM, N_("_Start"), NULL, play_start_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/start.png"), GDK_X, 0, NULL },
    { GNOME_APP_UI_ITEM, N_("S_top"), NULL, play_stop_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/stop.png"), GDK_C, 0, NULL },
    { GNOME_APP_UI_TOGGLEITEM, N_("P_ause"), NULL, play_pause_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/pause.png"), GDK_V, 0, NULL },
    { GNOME_APP_UI_ITEM, N_("_Next"), NULL, play_next_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/next.png"), GDK_B, 0, NULL },
    { GNOME_APP_UI_ITEM, N_("Skip 5sec forward"), NULL, play_keyseek_activate_cb,GINT_TO_POINTER(SEEK_OFFSET), NULL, GNOME_APP_PIXMAP_NONE, NULL, GDK_Right, 0, NULL },
    { GNOME_APP_UI_ITEM, N_("Skip 5sec backward"), NULL, play_keyseek_activate_cb,GINT_TO_POINTER(-SEEK_OFFSET), NULL, GNOME_APP_PIXMAP_NONE, NULL, GDK_Left, 0, NULL },
    GNOMEUIINFO_SEPARATOR,
    { GNOME_APP_UI_TOGGLEITEM, N_("Sh_uffle"), NULL, play_shuffle_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/shuffle.png"), GDK_U, GDK_CONTROL_MASK, NULL },
    { GNOME_APP_UI_TOGGLEITEM, N_("_Loop"), NULL, play_loop_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/loop.png"), GDK_L, GDK_CONTROL_MASK, NULL },
    GNOMEUIINFO_SEPARATOR,
    { GNOME_APP_UI_ITEM, N_("Volume _Down"), NULL, play_vol_down_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/vol_down.png"), GDK_comma, 0, NULL },
    { GNOME_APP_UI_ITEM, N_("_Volume Up"), NULL, play_vol_up_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/vol_up.png"), GDK_period, 0, NULL },
    { GNOME_APP_UI_TOGGLEITEM, N_("_Mute"), NULL, play_mute_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/vol_mute.png"), GDK_slash, 0, NULL },
    { GNOME_APP_UI_TOGGLEITEM, N_("_Effects"), NULL, play_effect_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, GDK_E, 0, NULL },
    GNOMEUIINFO_END
};

GnomeUIInfo view_menu_uiinfo[] = {
	GNOMEUIINFO_TOGGLEITEM(N_("_Toolbar"), NULL, view_toolbar_activate_cb, NULL),
    GNOMEUIINFO_TOGGLEITEM(N_("_Statusbar"), NULL, view_statusbar_activate_cb, NULL),
    GNOMEUIINFO_SEPARATOR,
    { GNOME_APP_UI_TOGGLEITEM, N_("Side_bar"), NULL, view_sidebar_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, GDK_F9, 0, NULL },
    GNOMEUIINFO_TOGGLEITEM(N_("Play _Info"), NULL, view_play_info_activate_cb, NULL),
    GNOMEUIINFO_TOGGLEITEM(N_("Play _Position"), NULL, view_play_pos_activate_cb, NULL),
    GNOMEUIINFO_SEPARATOR,
    { GNOME_APP_UI_ITEM, N_("_Refresh Playlist"), NULL, view_refresh_playlist_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, GDK_R, GDK_CONTROL_MASK, NULL },
    GNOMEUIINFO_END
};

GnomeUIInfo help_menu_uiinfo[] = {
    GNOMEUIINFO_HELP("liteamp"),
    GNOMEUIINFO_MENU_ABOUT_ITEM(help_about_activate_cb, NULL),
    GNOMEUIINFO_END
};

GnomeUIInfo menubar_uiinfo[] = {
    GNOMEUIINFO_MENU_FILE_TREE(file_menu_uiinfo),
    GNOMEUIINFO_MENU_EDIT_TREE(edit_menu_uiinfo),
    GNOMEUIINFO_SUBTREE(N_("_Play"), play_menu_uiinfo),
    GNOMEUIINFO_MENU_VIEW_TREE(view_menu_uiinfo),
    GNOMEUIINFO_MENU_HELP_TREE(help_menu_uiinfo),
    GNOMEUIINFO_END
};

/*-----------------------------------------------------------------*/

// main toolbar

GnomeUIInfo toolbar_uiinfo[] = {
    { GNOME_APP_UI_ITEM, N_("Previous"), NULL, toolbar_prev_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/prev.png"), 0, 0, NULL },
    { GNOME_APP_UI_ITEM, N_("Start"), NULL, toolbar_start_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/start.png"), 0, 0, NULL },
    { GNOME_APP_UI_ITEM, N_("Stop"), NULL, toolbar_stop_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/stop.png"), 0, 0, NULL },
    { GNOME_APP_UI_TOGGLEITEM, N_("Pause"), NULL, toolbar_pause_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/pause.png"), 0, 0, NULL },
    { GNOME_APP_UI_ITEM, N_("Next"), NULL, toolbar_next_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/next.png"), 0, 0, NULL },
    { GNOME_APP_UI_TOGGLEITEM, N_("Shuffle"), NULL, toolbar_shuffle_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/shuffle.png"), 0, 0, NULL },
    { GNOME_APP_UI_TOGGLEITEM, N_("Loop"), NULL, toolbar_loop_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/loop.png"), 0, 0, NULL },
	GNOMEUIINFO_END
};

/*-----------------------------------------------------------------*/

// sidebar popup

GnomeUIInfo sidebar_popup_uiinfo[] = {
    GNOMEUIINFO_MENU_PROPERTIES_ITEM(sidebar_props_activate_cb, NULL),
    GNOMEUIINFO_SEPARATOR,
    { GNOME_APP_UI_ITEM, N_("_New Playlist"), NULL, file_new_playlist_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_ADD, GDK_N, GDK_CONTROL_MASK, NULL },
    { GNOME_APP_UI_ITEM, N_("_Remove Playlist"), NULL, file_remove_playlist_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_REMOVE, 0, 0, NULL },
    { GNOME_APP_UI_ITEM, N_("Add _Files..."), NULL, file_add_file_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/addfile.png"), GDK_F, GDK_CONTROL_MASK, NULL },
    { GNOME_APP_UI_ITEM, N_("Add _Directories..."), NULL, file_add_dir_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/adddir.png"), GDK_D, GDK_CONTROL_MASK, NULL },
    GNOMEUIINFO_END
};

/*-----------------------------------------------------------------*/

// playlist popup

GnomeUIInfo playlist_popup_uiinfo[] = {
    { GNOME_APP_UI_ITEM, N_("_Start"), NULL, play_start_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/start.png"), 0, 0, NULL },
    { GNOME_APP_UI_ITEM, N_("S_top"), NULL, play_stop_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/stop.png"), 0, 0, NULL },
    { GNOME_APP_UI_TOGGLEITEM, N_("P_ause"), NULL, play_pause_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, PIXMAP_PATH("/pause.png"), 0, 0, NULL },
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_MENU_CUT_ITEM(edit_cut_activate_cb, NULL),
    GNOMEUIINFO_MENU_COPY_ITEM(edit_copy_activate_cb, NULL),
    GNOMEUIINFO_MENU_PASTE_ITEM(edit_paste_activate_cb, NULL),
    { GNOME_APP_UI_ITEM, N_("_Delete"), NULL, edit_clear_activate_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_CLEAR, GDK_DELETE, 0, NULL },
    GNOMEUIINFO_MENU_PROPERTIES_ITEM(playlist_props_activate_cb, NULL),
    GNOMEUIINFO_END
};

/*uiinfo.c*/
